//
//  TagDefinitionTests.swift
//
//
//  Created by Loris Perret on 06/12/2023.
//

import Foundation
import XCTest

@testable import ResgenSwift

final class TagDefinitionTests: XCTestCase {
 
    // MARK: - Match line
    
    func testMatchingTagDefinition() {
        // Given
        let line = "[definition_name]"
        
        // When
        let definition = TagDefinition.match(line)
        
        // Expect
        XCTAssertNotNil(definition)
        XCTAssertEqual(definition?.title, "definition_name")
    }
    
    func testNotMatchingTagDefinition() {
        // Given
        let line1 = "definition_name"
        let line2 = "[definition_name"
        let line3 = "definition_name]"
        
        // When
        let definition1 = TagDefinition.match(line1)
        let definition2 = TagDefinition.match(line2)
        let definition3 = TagDefinition.match(line3)
        
        // Expect
        XCTAssertNil(definition1)
        XCTAssertNil(definition2)
        XCTAssertNil(definition3)
    }
    
    // MARK: - Matching tags
    
    func testMatchingTags() {
        // Given
        let definition = TagDefinition(title: "definition_name")
        definition.tags = ["ios","iosonly","notranslation"]
        
        // When
        let match1 = definition.hasOneOrMoreMatchingTags(inputTags: ["ios"])
        let match2 = definition.hasOneOrMoreMatchingTags(inputTags: ["iosonly"])
        let match3 = definition.hasOneOrMoreMatchingTags(inputTags: ["notranslation"])
        
        
        // Expect
        XCTAssertTrue(match1)
        XCTAssertTrue(match2)
        XCTAssertTrue(match3)
    }
    
    func testNotMatchingTags() {
        // Given
        let definition = TagDefinition(title: "definition_name")
        definition.tags = ["ios","iosonly","notranslation"]
        
        // When
        let match1 = definition.hasOneOrMoreMatchingTags(inputTags: ["droid"])
        let match2 = definition.hasOneOrMoreMatchingTags(inputTags: ["droidonly"])
        let match3 = definition.hasOneOrMoreMatchingTags(inputTags: ["azerty"])
        
        // Expect
        XCTAssertFalse(match1)
        XCTAssertFalse(match2)
        XCTAssertFalse(match3)
    }
    
    // MARK: - Raw properties
    
    func testGeneratedRawPropertyScreen() {
        // Given
        let definition = TagDefinition(title: "definition_name")
        definition.path = "ecran_un/"
        definition.name = "Ecran un"
        definition.type = "screen"
        
        // When
        let propertyScreen = definition.getProperty()
        
        // Expect
        let expectScreen = """
            func logScreenEcranUn() {
                logScreen(name: "Ecran un", path: "ecran_un/")
            }
        """
        
        XCTAssertEqual(propertyScreen.adaptForXCTest(), expectScreen.adaptForXCTest())
    }
    
    func testGeneratedRawPropertyEvent() {
        // Given
        let definition = TagDefinition(title: "definition_name")
        definition.path = "ecran_un/"
        definition.name = "Ecran un"
        definition.type = "event"
        
        // When
        let propertyEvent = definition.getProperty()
        
        // Expect
        let expectEvent = """
            func logEventEcranUn() {
                logEvent(name: "Ecran un")
            }
        """
        
        XCTAssertEqual(propertyEvent.adaptForXCTest(), expectEvent.adaptForXCTest())
    }
    
    func testGeneratedRawStaticPropertyScreen() {
        // Given
        let definition = TagDefinition(title: "definition_name")
        definition.path = "ecran_un/"
        definition.name = "Ecran un"
        definition.type = "screen"
        
        // When
        let propertyScreen = definition.getStaticProperty()
        
        // Expect
        let expectScreen = """
            static func logScreenEcranUn() {
                logScreen(name: "Ecran un", path: "ecran_un/")
            }
        """
        
        XCTAssertEqual(propertyScreen.adaptForXCTest(), expectScreen.adaptForXCTest())
    }
    
    func testGeneratedRawStaticPropertyEvent() {
        // Given
        let definition = TagDefinition(title: "definition_name")
        definition.path = "ecran_un/"
        definition.name = "Ecran un"
        definition.type = "event"
        
        // When
        let propertyEvent = definition.getStaticProperty()
        
        // Expect
        let expectEvent = """
            static func logEventEcranUn() {
                logEvent(name: "Ecran un")
            }
        """
        
        XCTAssertEqual(propertyEvent.adaptForXCTest(), expectEvent.adaptForXCTest())
    }
}
